@@ -20,8 +20,7 @@
 #'   at the outlet of the levelpath the catchment is part of.
 #'   \item topo_sort is similar to Hydroseq in NHDPlus in that large topo_sort values
 #'   are upstream of small topo_sort values. Note that there are many valid topological
-#'   sort orders of a directed graph. The sort order output by this function is generated
-#'   using `igraph::topo_sort`.
+#'   sort orders of a directed graph.
 #' }
 #' @export
 #' @examples
@@ -41,19 +40,14 @@
 #'
 get_levelpaths <- function(x, override_factor = NULL, status = FALSE, cores = NULL) {
 
-  x <- check_names(x, "get_levelpaths")
+  x <- check_names(drop_geometry(x), "get_levelpaths")
 
-  x[["toID"]][which(is.na(x[["toID"]]))] <- 0
+  x[["toID"]] <- tidyr::replace_na(x[["toID"]], 0)
 
   x[["nameID"]][is.na(x[["nameID"]])] <- " " # NHDPlusHR uses NA for empty names.
   x[["nameID"]][x[["nameID"]] == "-1"] <- " "
 
-  sorted <- get_sorted(x)
-
-  sorted <- sorted[sorted != 0]
-
-  x <- left_join(data.frame(ID = as.numeric(sorted[!sorted == "NA"])),
-                 x, by = "ID")
+  x <- get_sorted(x)
 
   x[["topo_sort"]] <- seq(nrow(x), 1)
   x[["levelpath"]] <- rep(0, nrow(x))
@@ -69,7 +63,7 @@ get_levelpaths <- function(x, override_factor = NULL, status = FALSE, cores = NU
   }
 
   x <- x %>% # get downstream name ID added
-    left_join(select(x, .data$ID, ds_nameID = .data$nameID),
+    left_join(drop_geometry(select(x, .data$ID, ds_nameID = .data$nameID)),
               by = c("toID" = "ID")) %>%
     # if it's na, we need it to be an empty string
     mutate(ds_nameID = ifelse(is.na(.data$ds_nameID),
@@ -118,7 +112,9 @@ get_levelpaths <- function(x, override_factor = NULL, status = FALSE, cores = NU
   while(done < nrow(x) & checker < 10000000) {
     tail_topo <- outlets$topo_sort
 
-    pathIDs <- if(!is.null(cl)) {
+    pathIDs <- if(nrow(outlets) == 1) {
+      list(par_get_path(as.list(outlets), x, matcher, status))
+    } else if(!is.null(cl)) {
       parallel::parApply(cl = cl,
                          outlets[sample(nrow(outlets)), ], 1, par_get_path,
                          x_in = x, matcher = matcher,
@@ -153,7 +149,7 @@ get_levelpaths <- function(x, override_factor = NULL, status = FALSE, cores = NU
 
   outlets <- x %>%
     group_by(.data$levelpath) %>%
-    filter(topo_sort == min(topo_sort)) %>%
+    filter(.data$topo_sort == min(.data$topo_sort)) %>%
     ungroup() %>%
     select(outletID = .data$ID, .data$levelpath)
 
@@ -163,9 +159,9 @@ get_levelpaths <- function(x, override_factor = NULL, status = FALSE, cores = NU
 }
 
 par_get_path <- function(outlet, x_in, matcher, status) {
-  out <- get_path(x = x_in, tailID = outlet[names(outlet) == "ID"],
+  out <- get_path(x = x_in, tailID = outlet[names(outlet) == "ID"][[1]],
                   matcher = matcher, status = status)
-  data.frame(ID = out, levelpath = rep(outlet[names(outlet) == "topo_sort"], length(out)))
+  data.frame(ID = out, levelpath = rep(outlet[names(outlet) == "topo_sort"][[1]], length(out)))
 }
 
 #' get level path
@@ -186,33 +182,38 @@ get_path <- function(x, tailID, matcher, status) {
   toID <- NULL
 
   while(keep_going) {
+    tryCatch({
+      next_tails <- x[matcher[[as.character(tailID)]], ]
 
-    next_tails <- x[matcher[[as.character(tailID)]], ]
+      if(nrow(next_tails) > 1) {
 
-    if(nrow(next_tails) > 1) {
+        next_tails <- next_tails[next_tails$weight == max(next_tails$weight), ]
 
-      next_tails <- next_tails[next_tails$weight == max(next_tails$weight), ]
+      }
 
-    }
+      if(nrow(next_tails) == 0) {
 
-    if(nrow(next_tails) == 0) {
+        keep_going <- FALSE
 
-      keep_going <- FALSE
+      }
 
-    }
+      if(tailID %in% tracker) stop(paste0("loop at", tailID))
 
-    if(tailID %in% tracker) stop(paste0("loop at", tailID))
+      tracker[counter] <- tailID
 
-    tracker[counter] <- tailID
+      counter <- counter + 1
 
-    counter <- counter + 1
+      tailID <- next_tails$ID
 
-    tailID <- next_tails$ID
-
-    if(status && counter %% 1000 == 0) message(paste("long mainstem", counter))
+      if(status && counter %% 1000 == 0) message(paste("long mainstem", counter))
+    }, error = function(e) {
+      stop(paste0("Error with outlet tailID ", tailID, "\n",
+                  "Original error was \n", e))
+    })
 
   }
 
+
   return(tracker[!is.na(tracker)])
 }
 
@@ -285,15 +286,198 @@ reweight <- function(x, ..., override_factor) {
 }
 
 .datatable.aware <- TRUE
+. <- fromid <- id <- NULL
+
+get_fromids <- function(index_ids, return_list = FALSE) {
+  index_ids <- data.table::as.data.table(index_ids)
+
+  froms <- merge(
+    index_ids[,list(id)],
+    data.table::setnames(index_ids, c("toid", "id"), c("id", "fromid")),
+    by = "id", all.x = TRUE
+  )
+
+  froms <- froms[,list(froms = list(c(fromid))), by = id]
+
+  froms_l <- lengths(froms$froms)
+  max_from <- max(froms_l)
+
+  # Convert list to matrix with NA fill
+  froms_m <- as.matrix(sapply(froms$froms, '[', seq(max_from)))
+
+  # NAs should be length 0
+  froms_l[is.na(froms_m[1, ])] <- 0
+
+  if(return_list) return(list(froms = froms_m, lengths = froms_l,
+                              froms_list = froms))
+
+  return(list(froms = froms_m, lengths = froms_l))
+
+}
+
+#' Get Sorted Network
+#' @description given a tree with an id and and toid in the
+#' first and second columns, returns a sorted and potentially
+#' split set of output.
+#'
+#' Can also be used as a very fast implementation of upstream
+#' with tributaries navigation. The full network from each
+#' outlet is returned in sorted order.
+#'
+#' @export
+#' @param x data.frame with an identifier and to identifier in the
+#' first and second columns.
+#' @param split logical if TRUE, the result will be split into
+#' independent networks identified by the id of their outlet. The
+#' outlet id of each independent network is added as a "terminalID"
+#' attribute.
+#' @param outlets same as id in x; if specified only the network
+#' emanating from these outlets will be considered and returned.
+#' @return data.frame containing a topologically sorted version
+#' of the requested network and optionally a terminal id.
+#' @examples
+#' source(system.file("extdata/new_hope_data.R", package = "nhdplusTools"))
+#'
+#' fpath <- get_tocomid(
+#'   dplyr::select(new_hope_flowline, COMID, FromNode, ToNode, Divergence, FTYPE,
+#'                 AreaSqKM, LENGTHKM, GNIS_ID)
+#' )
+#'
+#' head(fpath <- get_sorted(fpath, split = TRUE))
+#'
+#' fpath['sort_order'] <- 1:nrow(fpath)
+#'
+#' plot(fpath['sort_order'])
+#'
+get_sorted <- function(x, split = FALSE, outlets = NULL) {
+
+  class_x <- class(x)
+
+  x <- as.data.frame(x)
+
+  # nrow to reuse
+  n <- nrow(x)
+
+  # index for fast traversal
+  index_ids <- get_index_ids(x, innames = names(x)[1:2])
+
+  if(!is.null(outlets)) {
+    starts <- which(x[, 1] %in% outlets)
+  } else {
+    # All the start nodes
+    starts <- which(index_ids$toid == 0)
+  }
+
+  froms <- get_fromids(index_ids)
+
+  # Some vectors to track results
+  to_visit <- out <- rep(0, n)
+
+  if(split) {
+    set <- out
+    out_list <- rep(list(list()), length(starts))
+  }
+
+  # output order tracker
+  o <- 1
+  set_id <- 1
+
+  for(s in starts) {
+
+    # Set up the starting node
+    node <- s
+
+    # within set node tracker for split = TRUE
+    n <- 1
+    # v is a pointer into the to_visit vector
+    v <- 1
+
+    while(v > 0) {
+
+      # track the order that nodes were visited
+      out[node] <- o
+      # increment to the next node
+      o <- o + 1
+
+      if(split) {
+        set[n] <- node
+        n <- n + 1
+      }
+
+      # does nothing if froms_l[node] == 0
+
+      for(from in seq_len(froms$lengths[node])) {
+        if(!is.na(next_node <- froms$froms[from, node])) {
+          # Add the next node to visit to the tracking vector
+          to_visit[v] <- next_node
+          v <- v + 1
+        }}
+
+      # go to the last element added in to_visit
+      v <- v - 1
+      node <- to_visit[v]
+
+    }
+
+    if(split) {
+      out_list[[set_id]] <- x[set[1:(n - 1)], 1]
+      set_id <- set_id + 1
+    }
+  }
+
+  if(split) names(out_list) <- x[starts, 1]
+
+  ### rewrites x into the correct order. ###
+  if(o - 1 != nrow(x)) {
+    x <- x[which(out != 0), ]
+    out <- out[out != 0]
+  }
+
+  x <- x[order(out)[(o-1):1], ]
+
+  if(split) {
+
+    # this is only two columns
+    ids <- methods::as(names(out_list), class(x[1, 1]))
+
+    out_list <- data.frame(ids = ids) %>%
+      mutate(set = out_list) %>%
+      tidyr::unnest_longer(.data$set)
+
+    names(out_list) <- c("terminalID", names(x)[1])
+
+    ### adds grouping terminalID to x ###
+    x <- dplyr::left_join(x, out_list, by = names(x)[1])
+
+  }
+
+  if("sf" %in% class_x) {
+    try(x <- sf::st_sf(x))
+  }
+
+  return(x)
+}
 
 #' @noRd
-#' @param x data.frame if an identifier and to identifier in the
-#' first and second columns.
-#' @importFrom igraph topo_sort graph_from_data_frame
-get_sorted <- function(x) {
-  x <- graph_from_data_frame(x, directed = TRUE)
-  x <- topo_sort(x, mode = "out")
-  names(x)
+#' @param x data.frame with ID and toID (names not important)
+#' in the first and second columns.
+#' An "id" and "toid" (lowercase) will be added.
+get_index_ids <- function(x,
+                          innames = c("comid", "tocomid"),
+                          outnames = c("id", "toid")) {
+
+  if(!all(innames %in% names(x))) {
+    stop(paste(paste(innames, collapse = ", "), "must be in input or provided."))
+  }
+
+  out <- data.frame(id = seq(1, nrow(x)))
+
+  out["toid"] <- match(x[[innames[2]]], x[[innames[1]]], nomatch = 0)
+
+  names(out) <- outnames
+
+  out
+
 }
 
 #' @noRd
@@ -303,34 +487,25 @@ topo_sort_network <- function(x, reverse = TRUE) {
 
   if(any(x$ID == 0)) stop("ID 0 must not be present. It is used as the outlet ID.")
 
-  x$toID[is.na(x$toID)] <- 0
+  x[["toID"]] <- tidyr::replace_na(x[["toID"]], 0)
 
-  sorted <- as(get_sorted(x[, c("ID", "toID")]),
-               class(x$ID))
+  x <- get_sorted(x[, c("ID", "toID", names(x)[!names(x) %in% c("ID", "toID")])])
 
   if(reverse) {
-    sorted <- sorted[length(sorted):1]
+    x <- x[nrow(x):1, ]
   }
 
-  sorted <- sorted[(!sorted == 0)]
-
-  order <- match(sorted, x$ID)
-
-  x <- x[order, ]
-
   x[!is.na(x$ID), ]
 
 }
 
-#' Get Terminal ID
+#' Get Terminal ID (DEPRECATED)
 #' @description Get the ID of the basin outlet for each flowline.
+#' This function has been deprecated in favor of get_sorted.
 #' @param x two column data.frame with IDs and toIDs. Names are ignored.
 #' @param outlets IDs of outlet flowlines
 #' @export
 #' @return data.frame containing the terminal ID for each outlet
-#' @importFrom igraph dfs graph_from_data_frame V
-#' @importFrom sf st_drop_geometry
-#' @importFrom tidyr unnest
 #' @examples
 #' source(system.file("extdata", "walker_data.R", package = "nhdplusTools"))
 #'
@@ -343,35 +518,17 @@ topo_sort_network <- function(x, reverse = TRUE) {
 #'
 get_terminal <- function(x, outlets) {
 
-  g <- graph_from_data_frame(x, directed = TRUE)
+  message("Deprecated, use get_sorted.")
 
-  basins <- lapply(outlets, function(o, g) {
-    v <- V(g)[which(names(igraph::V(g)) == o)]
+  ordered_id <- x$ID
 
-    b <- dfs(g, v, neimode = "in", unreachable = FALSE)
+  x <- get_sorted(x, split = TRUE, outlets = outlets)
 
-    b <- names(b$order)
-    b[!is.na(b) & b !=0]
-
-  }, g = g)
-
-  basin_df <- data.frame(terminalID = outlets, stringsAsFactors = FALSE)
-  basin_df[["ID"]] <- basins
-
-  basin_df <- distinct(unnest(basin_df, cols = "ID"))
-
-  if(is.integer(x[[1, 1]])) {
-    basin_df[["ID"]] <- as.integer(basin_df[["ID"]])
-  }
-
-  if(is.numeric(x[[1, 1]]) & !is.integer(x[[1, 1]])) {
-    basin_df[["ID"]] <- as.numeric(basin_df[["ID"]])
-  }
-
-  return(basin_df)
+  dplyr::left_join(data.frame(ID = ordered_id),
+                   x[c("ID", "terminalID")], by = "ID")
 }
 
-#' Get path length
+#' Get Path Length
 #' @description Generates the main path length to a basin's
 #' terminal path.
 #' @param x data.frame with ID, toID, length columns.
